@@ -1,19 +1,19 @@
 -- Copyright 2009-2010 Corey O'Connor
-module Graphics.Vty.Debug ( module Graphics.Vty.Debug
-                          , module Graphics.Vty.Debug.Image
-                          )
+module Graphics.Vty.Debug
+  ( module Graphics.Vty.Debug
+  , module Graphics.Vty.Debug.Image
+  )
 where
 
-import Graphics.Vty.Prelude
-
 import Graphics.Vty.Attributes
+import Graphics.Vty.Image (DisplayRegion)
 import Graphics.Vty.Debug.Image
 import Graphics.Vty.Span
 
-import qualified Data.Vector as Vector 
+import qualified Data.Vector as Vector
 
 rowOpsEffectedColumns :: DisplayOps -> [Int]
-rowOpsEffectedColumns ops 
+rowOpsEffectedColumns ops
     = Vector.toList $ Vector.map spanOpsEffectedColumns ops
 
 allSpansHaveWidth :: DisplayOps -> Int -> Bool
@@ -23,7 +23,7 @@ allSpansHaveWidth ops expected
 spanOpsEffectedRows :: DisplayOps -> Int
 spanOpsEffectedRows ops
     = toEnum $ length (filter (not . null . Vector.toList) (Vector.toList ops))
-        
+
 type SpanConstructLog = [SpanConstructEvent]
 data SpanConstructEvent = SpanSetAttr Attr
 
@@ -37,4 +37,3 @@ data MockWindow = MockWindow Int Int
 
 regionForWindow :: MockWindow -> DisplayRegion
 regionForWindow (MockWindow w h) = (w,h)
-
