@@ -1,15 +1,12 @@
 {-# OPTIONS_HADDOCK hide #-}
--- This makes a kind of tri. Has space efficiency issues with large input blocks.
--- Likely building a parser and just applying that would be better.
--- I did not write this so I might just rewrite it for better understanding. Which is not the best
--- of reasons.
--- TODO: measure and rewrite if required.
--- TODO: The ClassifyMap interface requires this code to always assure later entries override
--- earlier.
+-- This makes a kind of tri. Has space efficiency issues with large
+-- input blocks. Likely building a parser and just applying that would
+-- be better.
 module Graphics.Vty.Input.Classify
-    ( classify
-    , KClass(..)
-    ) where
+  ( classify
+  , KClass(..)
+  )
+where
 
 import Graphics.Vty.Input.Events
 import Graphics.Vty.Input.Mouse
@@ -34,23 +31,23 @@ compile table = cl' where
     eventForInput = M.fromList table
     cl' [] = Prefix
     cl' inputBlock = case M.lookup inputBlock eventForInput of
-            -- if the inputBlock is exactly what is expected for an event then consume the whole
-            -- block and return the event
+            -- if the inputBlock is exactly what is expected for an
+            -- event then consume the whole block and return the event
             Just e -> Valid e []
             Nothing -> case S.member inputBlock prefixSet of
                 True -> Prefix
-                -- look up progressively smaller tails of the input block until an event is found
-                -- The assumption is that the event that consumes the most input bytes should be
-                -- produced.
+                -- look up progressively smaller tails of the input
+                -- block until an event is found The assumption is that
+                -- the event that consumes the most input bytes should
+                -- be produced.
                 -- The test verifyFullSynInputToEvent2x verifies this.
-                -- H: There will always be one match. The prefixSet contains, by definition, all
-                -- prefixes of an event. 
+                -- H: There will always be one match. The prefixSet
+                -- contains, by definition, all prefixes of an event.
                 False ->
                     let inputPrefixes = reverse $ take maxValidInputLength $ tail $ inits inputBlock
                     in case mapMaybe (\s -> (,) s `fmap` M.lookup s eventForInput) inputPrefixes of
                         (s,e) : _ -> Valid e (drop (length s) inputBlock)
                         -- neither a prefix or a full event.
-                        -- TODO: debug log
                         [] -> Invalid
 
 classify :: ClassifyMap -> [Char] -> KClass
@@ -76,7 +73,8 @@ classifyUtf8 c cs =
   in case decode codepoint8 of
        _ | n < length codepoint -> Prefix
        Just (unicodeChar, _)    -> Valid (EvKey (KChar unicodeChar) []) rest
-       Nothing                  -> Invalid -- something bad happened; just ignore and continue.
+       -- something bad happened; just ignore and continue.
+       Nothing                  -> Invalid
 
 utf8Length :: (Num t, Ord a, Num a) => a -> t
 utf8Length c
