@@ -1,12 +1,10 @@
-{-# LANGUAGE CPP #-}
-
-#ifndef MIN_VERSION_base
-#defined MIN_VERSION_base(x,y,z) 1
-#endif
-
 -- Copyright 2009-2010 Corey O'Connor
-module Graphics.Vty.Output.XTermColor ( reserveTerminal )
-    where
+-- | Xterm output driver. This uses the Terminfo driver with some
+-- extensions for Xterm.
+module Graphics.Vty.Output.XTermColor
+  ( reserveTerminal
+  )
+where
 
 import Graphics.Vty.Output.Interface
 import Graphics.Vty.Input.Mouse
@@ -23,21 +21,16 @@ import System.Posix.IO (fdWrite)
 import System.Posix.Types (Fd)
 import System.Posix.Env (getEnv)
 
-#if !(MIN_VERSION_base(4,8,0))
-import Control.Applicative
-import Data.Foldable (foldMap)
-#endif
-
 import Data.List (isInfixOf)
 import Data.Maybe (catMaybes)
 import Data.Monoid ((<>))
 
--- | Initialize the display to UTF-8. 
+-- | Construct an Xterm output driver. Initialize the display to UTF-8.
 reserveTerminal :: ( Applicative m, MonadIO m ) => String -> Fd -> m Output
 reserveTerminal variant outFd = liftIO $ do
     let flushedPut = void . fdWrite outFd
-    -- If the terminal variant is xterm-color use xterm instead since, more often than not,
-    -- xterm-color is broken.
+    -- If the terminal variant is xterm-color use xterm instead since,
+    -- more often than not, xterm-color is broken.
     let variant' = if variant == "xterm-color" then "xterm" else variant
 
     utf8a <- utf8Active
@@ -100,16 +93,15 @@ disableBracketedPastes = "\ESC[?2004l"
 
 -- | These sequences set xterm based terminals to UTF-8 output.
 --
--- \todo I don't know of a terminfo cap that is equivalent to this.
+-- There is no known terminfo capability equivalent to this.
 setUtf8CharSet, setDefaultCharSet :: String
 setUtf8CharSet = "\ESC%G"
 setDefaultCharSet = "\ESC%@"
 
--- | I think xterm is broken: Reseting the background color as the first bytes serialized on a
--- new line does not effect the background color xterm uses to clear the line. Which is used
--- *after* the next newline.
+-- | I think xterm is broken: Reseting the background color as the first
+-- bytes serialized on a new line does not effect the background color
+-- xterm uses to clear the line. Which is used *after* the next newline.
 xtermInlineHack :: Output -> IO ()
 xtermInlineHack t = do
     let writeReset = foldMap (writeWord8.toEnum.fromEnum) "\ESC[K"
     outputByteBuffer t $ writeToByteString writeReset
-
